#!/bin/env python3
import os
import textwrap
import json


def read_file():
    _link_mappings = {}
    with open("vanitymap.txt", "r") as file:
        for line in file:
            if line.startswith("#") or len(line.strip()) == 0:
                continue
            _path, _url, *_ = line.split()
            if _path in _link_mappings:
                raise ValueError(f"Redirect target for {_path} already defined")
            _link_mappings[_path] = _url
    return _link_mappings


def flatten_links(_link_mappings):
    remaining_rounds = 10
    while remaining_rounds != 0:
        found = False
        for key, value in _link_mappings.items():
            if value and value.startswith("/"):
                found = True
                print(f"Detected internal link: {key} -> {value}")
                if value not in _link_mappings.keys():
                    print(f"ERROR: Target {value} not found!")
                    _link_mappings[key] = None
                else:
                    print(f"Replacing {value} with {_link_mappings[value]}")
                    _link_mappings[key] = _link_mappings[value]
        remaining_rounds = remaining_rounds - 1
        if not found:
            print("No (remaining) internal links found, quitting loop")
            remaining_rounds = 0


def write_redirect_html(sub_path, target_url):
    out_path = os.path.join("public", sub_path.lstrip(os.path.sep), "index.html")
    os.makedirs(os.path.dirname(out_path), exist_ok=True)
    with open(out_path, "w") as file:
        content = f'''\
        <!DOCTYPE html>
        <html>
            <head>
                <meta charset="UTF-8">
                <title>Du wirst weitergeleitet&hellip;</title>
                <link rel="canonical" href="{target_url}">
                <script>location="{target_url}"</script>
                <meta http-equiv="refresh" content="0; url={target_url}">
                <meta name="robots" content="noindex">
            </head>
            <body>
                <h1>Du wirst weitergeleitet&hellip;</h1>
                Du wirst zu <a href="{target_url}">{target_url}</a> weitergeleitet.
            </body>
        </html>
        '''
        file.write(textwrap.dedent(content))

def export_vanitymap(link_mappings, out_path):
    serialized_map = json.dumps(link_mappings)
    os.makedirs(os.path.dirname(out_path), exist_ok=True)
    with open(out_path, "w") as file:
        file.write(serialized_map)


if __name__ == '__main__':
    link_mappings = read_file()
    flatten_links(link_mappings)
    export_vanitymap(link_mappings, "public/vanitymap.json")
    for path, url in link_mappings.items():
        if url:
            print(f"Writing redirect for {path}")
            write_redirect_html(path, url)
